/*
 * Created on 27.04.2004
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;
import java.io.*;
import javax.xml.parsers.*;
import javax.xml.parsers.SAXParser;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.*;
import org.w3c.dom.*;


class XMLFilter implements FilenameFilter {
    public boolean accept(File dir, String name) {
        return (name.endsWith(".xml"));
    }
}

class SaveXMLFilter extends javax.swing.filechooser.FileFilter {
	public boolean accept(File dir) 
	{
        return (dir.canWrite() && dir.getName().endsWith(".xml"));
    }
    
    public String getDescription() { return "XML files"; }
}

/**
 * This class ...
 * @author knoeohcd
 */
public class JFractalView extends Frame implements ActionListener,ChangeListener {
	
	public static void main(String[] args) 
	{
		JFractalView mainwindow = new JFractalView();
		mainwindow.setVisible(true);
	}
	
	public JFractalView()
	{
	  setTitleText();
	  setSize(400,400);
	  MenuBar menubar = new MenuBar();
	  Menu filemenu = new Menu("File");
      filemenu.add(fileNew = new MenuItem("New"));
	  filemenu.add(fileOpen = new MenuItem("Open"));
	  filemenu.add(fileSave = new MenuItem("Save"));
	  filemenu.add(fileSaveAs = new MenuItem("Save as"));
	  filemenu.addSeparator();
	  filemenu.add(screenShot = new MenuItem("ScreenShot"));
	  filemenu.addSeparator();
	  filemenu.add(fileQuit = new MenuItem("Quit"));
	  menubar.add(filemenu);
	  Menu viewMenu = new Menu("View");
	  viewMenu.add(viewResetZoom = new MenuItem("Reset zoom"));
	  viewMenu.add(viewZoomIn = new MenuItem("Zoom in"));
	  viewMenu.add(viewZoomOut = new MenuItem("Zoom out"));
	  viewMenu.add(viewSetZoom = new MenuItem("Set zoom"));
	  menubar.add(viewMenu);
	  Menu helpMenu = new Menu("Help");
	  helpMenu.add(helpAbout=new MenuItem("About"));
	  menubar.setHelpMenu(helpMenu);
	  setMenuBar(menubar);
	  
	  fileNew.setShortcut(new MenuShortcut(KeyEvent.VK_N,false));
	  fileNew.addActionListener(this);
	  fileQuit.setShortcut(new MenuShortcut(KeyEvent.VK_Q,false));
	  fileQuit.addActionListener(this);
	  fileOpen.setShortcut(new MenuShortcut(KeyEvent.VK_O,false));
	  fileOpen.addActionListener(this);
	  screenShot.setShortcut(new MenuShortcut(KeyEvent.VK_PRINTSCREEN,false));
	  screenShot.addActionListener(this);
	  fileSave.setShortcut(new MenuShortcut(KeyEvent.VK_S,false));
	  fileSave.addActionListener(this);
	  fileSaveAs.addActionListener(this);
	  viewResetZoom.setShortcut(new MenuShortcut(KeyEvent.VK_BACK_SPACE,false));
	  viewResetZoom.addActionListener(this);
	  viewZoomIn.setShortcut(new MenuShortcut(KeyEvent.VK_PLUS,false));
	  viewZoomIn.addActionListener(this);
	  viewZoomOut.setShortcut(new MenuShortcut(KeyEvent.VK_MINUS,false));
	  viewZoomOut.addActionListener(this);
	  viewSetZoom.setShortcut(new MenuShortcut(KeyEvent.VK_ENTER,false));
	  viewSetZoom.addActionListener(this);
	  helpAbout.setShortcut(new MenuShortcut(KeyEvent.VK_F1,false));
	  helpAbout.addActionListener(this);
	  
	  setLayout(new BorderLayout());
	  statusBar=new Label("ready",Label.LEFT);
	  add(view=new FractalViewWidget(this),BorderLayout.CENTER);
	  Panel buttonpanel = new Panel();
	  buttonpanel.setLayout(new BorderLayout(5,5));
	  buttonpanel.add(statusBar,BorderLayout.WEST);
	  buttonpanel.add(itdepthspinner = new JSpinner(),BorderLayout.EAST);
	  itdepthspinner.setToolTipText("Iteration Depth");
	  itdepthspinner.setValue(new Integer(view.getIterationDepth()));
	  itdepthspinner.addChangeListener(this);
	  add(buttonpanel,BorderLayout.SOUTH);
	  pack();
	}
	
  public void actionPerformed(ActionEvent e)
  {
   Object source = e.getSource();
   if(source == fileQuit) quitAction();
   else if(source == fileNew) newAppAction();
   else if(source == fileOpen) openAction();
   else if(source == fileSave) saveAction();
   else if(source == fileSaveAs) saveAsAction();
   else if(source == screenShot) view.takeScreenShot();
   else if(source == viewResetZoom) view.resetView();
   else if(source == viewZoomIn) view.zoomIn();
   else if(source == viewZoomOut) view.zoomOut();
   else if(source == viewSetZoom) setZoomAction();
   else if(source == helpAbout) helpAboutAction();
  }

  public void stateChanged(ChangeEvent e)
  {
  	// there is only one possible source at the moment...
  	int value= ((Integer) itdepthspinner.getValue()).intValue();
  	if(value < 0) itdepthspinner.setValue(new Integer(200));
  	else view.setIterationDepth(value);
  }
  
  /**
   * check for unsaved changes and quits the application
   */
  final private void quitAction()
  {
  	dispose();
  	System.exit(0);
  }
  
  final private void newAppAction()
  {
  	JFractalView mainwindow = new JFractalView();
	mainwindow.setVisible(true);
  }

  final private void openAction()
  {
  	FileDialog dlg = new FileDialog(this,"Open a new fractal description.",FileDialog.LOAD);
  	dlg.setFilenameFilter(new XMLFilter());
  	dlg.setVisible(true);
  	String filename = dlg.getDirectory()+dlg.getFile();
  	if(filename != null)
  	{
  		try
		{
  			SAXParserFactory factory = SAXParserFactory.newInstance();
  			SAXParser saxParser = factory.newSAXParser();
  			FractalSAXParser handler = new FractalSAXParser();
  			saxParser.parse( new File(filename), handler);
  			view.setViewedArea(handler.ul_real,handler.ul_imag,handler.lr_real,handler.lr_imag,handler.iteration_depth);
  			itdepthspinner.setValue(new Integer(handler.iteration_depth));
  			fileName=filename;
  			setTitleText();
		}
  		catch(Throwable e)
		{
  			JOptionPane.showMessageDialog(this,"Error while opening file:\n"+e.getMessage(),"JFractalView",JOptionPane.ERROR_MESSAGE);
  		}
  	}
  }
  
  final private void helpAboutAction()
  {
  	JOptionPane.showMessageDialog(this,"JFractalViewer\nGPL-Software hacked for you by Ruediger Knoerig.","JFractalView",JOptionPane.INFORMATION_MESSAGE);
  }
  
  final private void setTitleText()
  {
  	setTitle("JFractalView "+((fileName == null) ? "new File" : fileName));
  }
  
  final private void saveAction()
  {
  	if(fileName == null) saveAsAction();
  	else
  	{
  		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
  	    try {
  	      DocumentBuilder builder = factory.newDocumentBuilder();
  	      Document document = builder.newDocument();
  	      Element root = (Element) document.createElement("mandelbrot_set"); 
  	      document.appendChild(root);
  	      Element itdepth = (Element) document.createElement("iteration_depth");
  	      root.appendChild(itdepth);
  	      itdepth.appendChild(document.createTextNode(String.valueOf(view.getIterationDepth())));
  	      //itdepth.setNodeValue();
  	      Element upperleft = (Element) document.createElement("upper_left");
  	      upperleft.setAttribute("real",String.valueOf(view.getUpperLeftReal()));
  	      upperleft.setAttribute("imag",String.valueOf(view.getUpperLeftImag()));
  	      root.appendChild(upperleft);
  	      Element lowerright = (Element) document.createElement("lower_right");
  	      lowerright.setAttribute("real",String.valueOf(view.getLowerRightReal()));
  	      lowerright.setAttribute("imag",String.valueOf(view.getLowerRightImag()));
  	      root.appendChild(lowerright);
 		  TransformerFactory tFactory = TransformerFactory.newInstance();
   		  Transformer transformer = tFactory.newTransformer();

   		  DOMSource source = new DOMSource(document);
          StreamResult result = new StreamResult(new BufferedWriter(new FileWriter(fileName)));
          transformer.transform(source, result);
 		  setTitleText();
  	    }
  	 catch (ParserConfigurationException pce) 
	 {
  	 	// Parser with specified options can't be built
        JOptionPane.showMessageDialog(this,"DOM Parser configuration error:\n"+pce.getMessage(),"JFractalView",JOptionPane.ERROR_MESSAGE); 
     } 
  	 catch (IOException ioe) 
	 {
        // I/O error
        JOptionPane.showMessageDialog(this,"File I/O error:\n"+ioe.getMessage(),"JFractalView",JOptionPane.ERROR_MESSAGE);
     }
     catch (TransformerConfigurationException tce) 
     {
   		// Error generated by the parser
   		JOptionPane.showMessageDialog(this,"Error while preparing XML builder:\n"+tce.getMessage(),"JFractalView",JOptionPane.ERROR_MESSAGE);
	 } 
	catch (TransformerException te) 
	 {
   		// Error generated by the parser
        JOptionPane.showMessageDialog(this,"Error while building XML file:\n"+te.getMessage(),"JFractalView",JOptionPane.ERROR_MESSAGE);     
	 }
  	}
  }
  
  final private void saveAsAction()
  {
  	JFileChooser dlg = new JFileChooser(fileName);
  	dlg.setFileFilter(new SaveXMLFilter());
  	if(dlg.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
  	{
  		fileName = dlg.getSelectedFile().getAbsolutePath();
  		saveAction();
  	}
  }
  
  final private void setZoomAction()
  {
  	SelectViewedAreaDialog dlg = new SelectViewedAreaDialog(this,
  							   								view.getLowerRightReal(),
  							   								view.getLowerRightImag(),
  							   								view.getUpperLeftReal(),
  							   								view.getUpperLeftImag());
    if(dlg.showDialog())
    {
    	view.setViewedArea(dlg.getLowerRightReal(),
    					   dlg.getLowerRightImag(),
    					   dlg.getUpperLeftReal(),
    					   dlg.getUpperLeftImag(),
    					   view.getIterationDepth());
    }  							   								
  }
  
  private MenuItem fileNew;
  private MenuItem fileOpen;
  private MenuItem fileSave;
  private MenuItem fileSaveAs;
  private MenuItem fileQuit;
  private MenuItem screenShot;
  private MenuItem viewResetZoom;
  private MenuItem viewZoomIn;
  private MenuItem viewZoomOut;
  private MenuItem viewSetZoom;
  private MenuItem helpAbout;
  public Label statusBar;
  private FractalViewWidget view;
  private String fileName;
  private JSpinner itdepthspinner;
}
